#! /usr/bin/env python
#Fri November 26th 2010
#python script designed to take a input file that is an IRC log and
#create an HTML file that can be uploaded to an appropriate web server for
#consumption.



#version history


#1.1: fixed documentation error in readme, and added -i and -ignoreextensions switch.


#1.00: Initial release.


#ConfigParser: for parsing the INI file; os for file manip, and re because
#I got "lazy" with the fixlinks routine.
import ConfigParser,os,re
#and optionparser for parsing the command line.
from optparse import OptionParser




#replaceit is used by "fixlinks" when it calls re.sub to perform a
#regular expression substitution. I tried to figure out how to
#work with backreferences but I couldn't get it to work, but this works fine.
def replaceit(match):
    return "<a href=\"" + match.group() + "\">" + match.group() + "</a>" 

#function called to convert urls into links.
def fixlinks(fullstr):
    global options,args

    #regpattern=r"(http|https|ftp|file)://[-A-Z0-9+&@#/%?=~_|!:,.;]*[A-Z0-9+&@#/%=~_|]"
    regpattern=r"(?im)\b(https|http|ftp|file)://[-A-Z0-9+&@#/%?=~_|!:,.;]*[A-Z0-9+&@#/%=~_|]"
    replacewith=replaceit

    return re.sub(regpattern,replacewith,fullstr,re.IGNORECASE | re.MULTILINE)



#takes the given input irc logfile infile, and creates a formatted IRC log in outfile.
def irc2html(infile,outfile,stylesheet,config):

    global args,options #need to work with the global options value...



    print "converting input file \"" + infile + "\" to HTML in output file:\"" + outfile + "\""
    fileopened=open(infile)
    fileout=open(outfile,"w")
    #write out the basic header.
    fileout.write("<html>\n")
    fileout.write("\t<head>\n")
    fileout.write("\t<style type=\"text/css\">")
    print "opening stylesheet, " + stylesheet
    csscode=open(stylesheet).read()
    
    print "performing global find/replace on CSS code..."
    csscode=replacecss(csscode,config)
    fileout.write(csscode + "\n")
    fileout.write("\t</style>\n")
    fileout.write("\t<title></title>\n")
    fileout.write("\t</head>\n")
    fileout.write("<body bgcolor=" + config["bgcolor"] + " text=" + config["text"] +
                  " link=" + config["link"] + " vlink=" + config["vlink"] + " alink=" + 
                  config["alink"] + ">\n")
                  
    slurped=""   
                  
    
    for inputline in fileopened:
        if len(inputline)>10:
            if inputline[8]=="<":
                firstangle=inputline.find("<")
                secondangle=inputline.find(">",firstangle)
                foundnick=inputline[firstangle+1:secondangle-1]
                
                foundnick=colourizenick(foundnick)
                inputline=inputline[0:firstangle-1] + "&#60;" + foundnick.strip() + "&#62;" + inputline[secondangle+1:].replace(">","&#62;").replace("<","&#60;") + "<br>"
            elif inputline[8]=="*":
                foundspace=inputline.find(" ",11)
                foundnick=inputline[10:foundspace].lower()
                if options.verbose:
                   print "foundnick=" + foundnick
                if foundnick=="chanserv":
                    inputline=inputline[:8] + "<span class=\"CHANSERV\">" + inputline[8:] + "</span><br>"
                else:
                    inputline=inputline[:8] + "<span class=\"ACTION\">" + inputline[8:] + "</span><br>"
            
            
            
            if inputline.startswith("["):
                #colourize time.
                firstsquare=inputline.find("[")
                secondsquare=inputline.find("]",firstsquare)
                foundtime=inputline[firstsquare:secondsquare+1]
                if options.verbose:    
                    print "foundtime=" + foundtime
                inputline=inputline[:firstsquare] + "<span class=\"TIME\">" + foundtime + "</span>" + inputline[secondsquare+1:]
            
            slurped=slurped+inputline+ "\n"
        
    #end of for loop
        #fixlinks calls would go here
    
    if options.upmarkurls:
        slurped=fixlinks(slurped)
    if options.hardrulesessions:    
        slurped=hardrulesessions(slurped)
    #replace all carriage returns with <br>...    

    while slurped.find("<br>\n<br>")> -1:
        slurped=slurped.replace("<br>\n<br>","<br>\n")
    

        
    #while slurped.find("<br><br>") > -1:
    #    slurped=slurped.replace("<br><br>","<br>")        
        
    fileout.write(slurped)
    fileout.write("</body></html>")

def hardrulesessions(fulltext):
    #find "\nSession Start:" and replace with "<hr>\nSession Start:"
    return fulltext.replace("\nSession Start:","<hr>\nSession Start:")


#routine for colourizing a given nickname. 
def colourizenick(nick):
    global options,args
    nick=nick.strip()
    if options.verbose:
        print "colourizing nickname: \"" + nick + "\""
    startstr="<span clas=\"USER\">"
    endstr="</span>"
    if nick[0]=="~" or nick.find("GlitchPC")>-1:
        startstr="<span class=\"FOUNDER\">"
        endstr = "</span>"
    elif nick[0]=="&":
        startstr="<span class=\"COADMIN\">"
        endstr = "</span>"
    elif nick[0]=="@":
        startstr="<span class=\"OP\">"
        endstr="</span>"
    elif nick.startswith("@Chanbot"):
        startstr="<span class=\"CHANBOT\">"
        endstr="</span>"
    else:
        startstr="<span class=\"USER\">"
        
    return startstr + nick.strip() + endstr

#findnicks: scans a file and locates all irc nicknames, returning a pleasant list.
def findnicks(scantext):
    #first, create the list we will use to store values:
    returnlist=list()
    #now, we need to open the file
    filescan=open(scantext)
    #iterate through every line in the file
    for loopline in filescan:
        #for our purposes, the characters will be in very specific locations.
        if loopline[8]=="<":
            #this line contains a nickname...
            #grab the first angle bracket,
            firstangle=loopline.find("<")
            #and the second angle bracket..
            secondangle=loopline.find(">",firstangle+1)
            #slice out the nickname...
            foundnick=loopline[firstangle:secondangle]
            #and add it to our list.
            returnlist.append(foundnick)
            
    return returnlist




#replaceCSS: replaces the specified text with the various options in the given "options" dictionary.
def replacecss(csstext,useoptions):
    #replace special "markup" in the given csstext with the values from the 
    #given dictionary (useoptions).xt

    for replacekey in useoptions:
        print "replacing string: " + "%%" + replacekey.upper() + "%%" + " with " + useoptions[replacekey];
        csstext=csstext.replace("%%" + replacekey.upper() + "%%",useoptions[replacekey])
    


    return csstext;







parser = OptionParser(usage="usage: %prog [options] inputfile outputfile [CSSFile]",
                      version="%prog 1.0")
parser.add_option("-v","--verbose",
action="store_true", dest="verbose", default=False,
help="display extra (and useless) information during processing.")


parser.add_option("-u","--upmarkurls",action="store_true",dest="upmarkurls",default=False,
help="convert URLs in the file to links.")


parser.add_option("-r","--hardrulesessions",action="store_true",dest="hardrulesessions",default=False,
help="insert hard rules (<hr>'s) between log sessions (indicated by \"Session Start:\" at the beginning of a line)")


parser.add_option("-i","--ignoreextension",action="store_true",dest="ignoreextension",default=False,
help="instead of replacing the extension on the input file when no output file is specified, it simply appends .html to the name.")
global options,args
(options,args) = parser.parse_args();


config = ConfigParser.SafeConfigParser();
config.readfp(open('irc2html.cfg'))
statsfile=".\stats.p"
#initialize options....
opts={}
#
for configname,configitem in config.items('options'):
    opts[configname]=configitem


if options.verbose:
    print "using options:"
    print options   
    print "arguments:"
    print args
    print "opts:"
    print opts

if len(args)<1:
    exit("error: insufficient number of arguments.")
if len(args)>=2:
    inputfile=args[0]
    outputfile=args[1]
elif len(args)==1:
    #a single argument, a single filename... we need to create out own name.
    inputfile=args[0]
    if options.ignoreextension:
        outputfile = inputfile + ".html"
    else:
        outputfile = os.path.splitext(inputfile)[0] + ".html"
    
    
    
    
if len(args)>2:
   cssfile=args[2]
else:
   cssfile="./ircstyle.css"  #default to "built-in" css file.
#note to self: remember cssfiles cannot be "standard" CSS files, but should instead have the appropriate replacable parameters inside.


print len(args) 
irc2html(inputfile,outputfile,cssfile,opts)






